#  Copyright (c) ZenML GmbH 2023. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at:
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
#  or implied. See the License for the specific language governing
#  permissions and limitations under the License.

from zennews.steps.sources.bbc import bbc_news_source, BBCParameters
from zennews.steps.report.report import post_summaries
from zennews.steps.summarize.bart_large_cnn_samsum import bart_large_cnn_samsum

from zennews.materializers.article_materializer import ArticleMaterializer  # noqa

SOURCE_STEP_MAPPING = {
    "bbc": {"step": bbc_news_source, "parameters": BBCParameters}
}


__all__ = [
    "post_summaries",
    "bbc_news_source",
    "bart_large_cnn_samsum",
    "SOURCE_STEP_MAPPING"
]

